package timing.ukulele.auth.config;

import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.jwt.JwtDecoder;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.filter.CorsFilter;
import timing.ukulele.auth.config.property.TimingSecurityProperties;
import timing.ukulele.auth.security.filter.JWTSecurityFilter;
import timing.ukulele.auth.security.type.image.ImageCodeAuthenticationSecurityConfig;
import timing.ukulele.auth.security.type.sms.SmsCodeAuthenticationSecurityConfig;
import timing.ukulele.auth.security.type.wechat.ThirdQrAuthenticationSecurityConfig;

/**
 * 资源服务器配置
 * <p>
 * {@link EnableMethodSecurity} 开启全局方法认证，启用JSR250注解支持，启用注解 {@link Secured} 支持，
 * 在Spring Security 6.0版本中将@Configuration注解从@EnableWebSecurity, @EnableMethodSecurity, @EnableGlobalMethodSecurity
 * 和 @EnableGlobalAuthentication 中移除，使用这些注解需手动添加 @Configuration 注解
 * {@link EnableWebSecurity} 注解有两个作用:
 * 1. 加载了WebSecurityConfiguration配置类, 配置安全认证策略。
 * 2. 加载了AuthenticationConfiguration, 配置了认证信息。
 */
@Configuration
@EnableWebSecurity
@RequiredArgsConstructor
@EnableMethodSecurity(jsr250Enabled = true, securedEnabled = true)
public class SecurityConfig {

    /**
     * 不需要认证即可访问的路径
     */
    private final TimingSecurityProperties customSecurityProperties;
    private final SmsCodeAuthenticationSecurityConfig smsCodeAuthenticationSecurityConfig;
    private final ImageCodeAuthenticationSecurityConfig imageCodeAuthenticationSecurityConfig;
    private final ThirdQrAuthenticationSecurityConfig thirdQrAuthenticationSecurityConfig;

    /**
     * 配置认证相关的过滤器链(资源服务，客户端配置)
     *
     * @param http spring security核心配置类
     * @return 过滤器链
     * @throws Exception 抛出
     */
    @Bean
    public SecurityFilterChain defaultSecurityFilterChain(HttpSecurity http,
                                                        CorsFilter corsFilter,
                                                          JwtDecoder jwtDecoder) throws Exception {
        // 添加基础的认证配置
        http.addFilter(corsFilter);

        // 禁用 csrf 与 cors
        http.csrf(AbstractHttpConfigurer::disable);
//        http.cors(AbstractHttpConfigurer::disable);

        http.httpBasic(Customizer.withDefaults())
                .authorizeHttpRequests((authorize) -> authorize
                        // 放行静态资源和不需要认证的url
                        .requestMatchers(customSecurityProperties.getIgnoreUriList().toArray(new String[0])).permitAll()
                        .anyRequest().authenticated()
                )
                .formLogin(AbstractHttpConfigurer::disable);
//        http.sessionManagement(AbstractHttpConfigurer::disable);
        http.sessionManagement(session->session.sessionCreationPolicy(SessionCreationPolicy.STATELESS));

        http.apply(smsCodeAuthenticationSecurityConfig);
        http.apply(imageCodeAuthenticationSecurityConfig);
        http.apply(thirdQrAuthenticationSecurityConfig);


        JWTSecurityFilter jwtFilter = new JWTSecurityFilter(jwtDecoder);
        http.addFilterAt(jwtFilter, UsernamePasswordAuthenticationFilter.class);

        DefaultSecurityFilterChain build = http.build();
        return build;
    }
}
